<?php
App::uses('EvShopAppController', 'EvShop.Controller');
App::uses('ArrayUtil', 'EvCore.Lib');

class CategoriesController extends EvShopAppController {

	/**
	 * redefine to allow the index / view page through the auth component!
	 * and also add the reorder product method
	 */
	public function beforeFilter() {
		$this->adminActions[] = 'admin_reorderProducts';

		parent::beforeFilter();

		$this->Auth->allow(array('index', 'view'));
	}

	/**
	 * categories index page
	 * list all categories out
	 *
	 */
	public function index() {
		die('123');
	}

	protected function _adminFilterFields() {
		$fields = parent::_adminFilterFields();

		unset(
			$fields['Category.id'],
			$fields['Category.created'],
			$fields['Category.modified']
		);

		return $fields;
	}

	protected function _adminIndexColumns() {
		$fields = parent::_adminIndexColumns();

		$fields = ArrayUtil::addAfter(
			$fields,
			'Category.name',
			array(
				'ParentCategory.name' => array(
					'type' => 'string',
					'label' => 'Parent',
					'default' => '-'
				)
			)
		);

		return $fields;
	}

	protected function _adminIndexPaginate() {
		$paginate = parent::_adminIndexPaginate();

		$paginate['contain'][] = 'ParentCategory';
		$paginate['order'] = 'Category.name ASC';

		return $paginate;
	}

	/**
	 * add the product reorder link
	 */
	protected function _adminIndexActions() {
		$actions = array(
			'Reorder' => array(
				'cell' => array(
					'class' => 'action pricing'
				),
				'link' => array(
					'url' => array(
						'action' => 'reorderProducts'
					),
					'text' => '<i class="fa fa-bars"></i>',
					'options' => array(
						'escape' => false
					)
				)
			)
		) + parent::_adminIndexActions();

		return $actions;
	}

	protected function _adminPopulateLookups() {
		parent::_adminPopulateLookups();

		$this->set(
			'parents',
			( // merge arrays with the + operator .... preserves numeric keys in arrays
				array('Top Level') +
				$this->{$this->modelClass}->generateTreeList(
					null,
					null,
					null,
					'--'
				)
			)
		);
	}

	/**
	 * reorder products within the category
	 */
	public function admin_reorderProducts($id) {
		$Model = $this->{$this->modelClass};

		if ($this->request->is('post') || $this->request->is('put')) {
			// format to allow us to save

			$save = array();
			foreach ($this->request->data['reorder'] as $item) {

				$save[] = array(
					'CategoriesProduct' => array(
						'id' => $item['id'],
						'sequence' => $item['sequence']
					)
				);
			}

			if ($Model->CategoriesProduct->saveMany($save)) {
				$this->Flash->success(
					array(
						'title' => 'Order Saved',
						'description' => 'Reordering has been saved successfully!'
					)
				);

				$this->redirect(
					array(
						'action' => 'index'
					)
				);
			} else {
				$this->Flash->fail(
					array(
						'title' => 'Reordering Failed',
						'description' => 'Unable to find an ordering column on the table'
					)
				);
			}
		}

		$data = $Model->CategoriesProduct->find('all', array(
			'conditions' => array(
				'CategoriesProduct.category_id' => $id
			),
			'order' => 'CategoriesProduct.sequence ASC',
			'contain' => array(
				'Product'
			)
		));

		$this->set(array(
			'order_key' => 'sequence',
			'max_depth' => 1,
			'data' => $data,
			'columns' => array(
				'CategoriesProduct.id' => true,
				'Product.name' => true
			)
		));

		$this->view = '/Scaffolds/admin_reorder';
	}
}
